;;  mine.el	(Version: @(#)mine.el 1.1 (99/01/22) for GNU Emacs)

;;     Author:  Minoru Tamura (tamura@dbg.bs1.fc.nec.co.jp)
;;	Please send suggestions and corrections to the above address.
;;
;;  This file contains mine-sweeper, a game named Mine Sweeper for GNU Emacs.

(defvar mnsw-flag 2)		; 1 for one-byte, 2 for two-byte map
(defvar mnsw-indv-dir "~")	; directory for individual history file
(defvar mnsw-grp-dir "~")	; directory for group history file
(defvar mnsw-mail-command "")	; mail command (/usr/ucblib/binmail)

(defvar mnsw-mode-map nil)

(if mnsw-mode-map
    nil
  (setq mnsw-mode-map (make-keymap))
  (define-key mnsw-mode-map " " 'mnsw-open)
  (define-key mnsw-mode-map "\C-m" 'mnsw-mine)
  (define-key mnsw-mode-map "\eq" 'mnsw-quit)
  (define-key mnsw-mode-map "\es" 'mnsw-save)
  (if (>= (string-to-int emacs-version) 19)
      (progn
	(define-key mnsw-mode-map [down] 'mnsw-nextline)
	(define-key mnsw-mode-map [up] 'mnsw-prevline)
	(define-key mnsw-mode-map [right] 'mnsw-forward)
	(define-key mnsw-mode-map [left] 'mnsw-backward)))
  (define-key mnsw-mode-map "\eOB" 'mnsw-nextline)
  (define-key mnsw-mode-map "\eOA" 'mnsw-prevline)
  (define-key mnsw-mode-map "\eOC" 'mnsw-forward)
  (define-key mnsw-mode-map "\eOD" 'mnsw-backward)
  (define-key mnsw-mode-map "\C-n" 'mnsw-nextline)
  (define-key mnsw-mode-map "\C-p" 'mnsw-prevline)
  (define-key mnsw-mode-map "\C-f" 'mnsw-forward)
  (define-key mnsw-mode-map "\C-b" 'mnsw-backward)
  (define-key mnsw-mode-map "j" 'mnsw-nextline)
  (define-key mnsw-mode-map "k" 'mnsw-prevline)
  (define-key mnsw-mode-map "l" 'mnsw-forward)
  (define-key mnsw-mode-map "h" 'mnsw-backward)
  (define-key mnsw-mode-map "x" 'mnsw-nextline)
  (define-key mnsw-mode-map "e" 'mnsw-prevline)
  (define-key mnsw-mode-map "d" 'mnsw-forward)
  (define-key mnsw-mode-map "s" 'mnsw-backward))

(defun mine-sweeper ()
  "Mine Sweeper"
  (interactive)
  (mnsw-quit)
  (random t)
  (let ((s (current-time-string)))
    (while (string= s (current-time-string))
      (random)))
  (setq mnsw-user (user-login-name))
  (if (> (length mnsw-user) 10)
      (setq mnsw-user (substring mnsw-user 0 10)))
  (setq mnsw-date (what-date))
  (setq mnsw-indv-file (concat mnsw-indv-dir "/.mine"))
  (setq mnsw-grp-file (concat mnsw-grp-dir "/.gmine"))
  (setq mnsw-save-file (concat mnsw-indv-dir "/mine.sv"))
  (setq mnsw-save-flag (file-exists-p mnsw-save-file))
  (setq mnsw-disp1 " %10s %4d(%3d) %4ds|")
  (setq mnsw-disp2 " %10s %3d %4ds|")
  (setq mnsw-disp3 " %4d(%3d) %4ds|")
  (setq mnsw-disp4 " %3d %4ds")
  (setq mnsw-title1
	"Mine Sweeper ver. 1.9 (99/01/14) Copywrong(C) 1992 by M. Tamura\n")
  (cond ((= mnsw-flag 1)
	 (setq mnsw-dotmark ".")
	 (setq mnsw-minemark "@")
	 (setq mnsw-question "?")
	 (setq mnsw-mark '("0" "1" "2" "3" "4" "5" "6" "7" "8" "x"))
	 (setq mnsw-lastmine '("x" "*" "+"))
	 (setq mnsw-title2 "%2dx%2d   x: %3d   Time %4ds   .: %3d/%3d   @: %3d/%3d"))
	((= mnsw-flag 2)
	 (setq mnsw-dotmark "")
	 (setq mnsw-minemark "")
	 (setq mnsw-question "")
	 (setq mnsw-mark '("" "" "" "" "" "" "" "" "" ""))
	 (setq mnsw-lastmine '("" "" ""))
	 (setq mnsw-title2 "%2dx%2d   : %3d   Time %4ds   : %3d/%3d   : %3d/%3d")))
  (if (boundp 'MULE)
      (setq mnsw-step 1)
    (setq mnsw-step mnsw-flag))
  (if mnsw-save-flag
      (mnsw-load)
    (mine1))
)

(defun mine1 ()
  (delete-other-windows)
  (kill-buffer-if-exists "*mine-history*")
  (switch-to-buffer "*mine-history*")
  (mnsw-disp-history (get-buffer "*mine-history*"))
  (let ((s nil))
    (while (not (or (string= s "q")
		    (string= s "1") (string= s "2") (string= s "3")))
      (setq s (read-string "Which size [1-3] or Quit [q]? ")))
    (if (string= s "q") 
	(mnsw-quit)
      (setq mnsw-size (string-to-int s))
      (setq mnsw-tate (* mnsw-size 6))
      (setq mnsw-yoko (* mnsw-size 10))
      (let ((v 0) (s nil))
	(while (or (<= v 0) (> v (/ (* mnsw-yoko mnsw-tate 2) 3)))
	  (setq s (read-string "How many mines? "))
	  (if (string= s "") (setq v 0)
	    (setq v (string-to-int s))))
	(setq mnsw-cnt v))
      (mine2)))
)

(defun mine2 ()
  (kill-buffer-if-exists "*mine*")
  (setq mnsw-map nil)
  (let ((l nil) (i 0))
    (while (< i mnsw-yoko)
      (setq l (cons 0 l))
      (setq i (1+ i)))
    (setq i 0)
    (while (< i mnsw-tate)
      (setq mnsw-map (cons l mnsw-map))
      (setq i (1+ i))))
  (let ((cnt 0) n x y)
    (while (< cnt mnsw-cnt)
      (setq n (random))
      (setq n (if (< n 0) (- n) n))
      (setq n (% n (* mnsw-yoko mnsw-tate)))
      (setq x (% n mnsw-yoko))
      (setq y (/ n mnsw-yoko))
      (if (= (mnsw-value x y) 9) t
	(mnsw-value-set x y 9)
	(let ((i -1) xi)
	  (while (<= i 1)
	    (setq xi (+ x i))
	    (if (or (< xi 0) (>= xi mnsw-yoko)) t
	      (let ((j -1) yj)
		(while (<= j 1)
		  (setq yj (+ y j))
		  (if (or (< yj 0) (>= yj mnsw-tate)) t
		    (let ((v (mnsw-value xi yj)))
		      (if (= v 9) t
			(mnsw-value-set xi yj (1+ v)))))
		  (setq j (1+ j)))))
	    (setq i (1+ i))))
	(setq cnt (1+ cnt)))))
  (mine3)
)

(defun mine3 ()
  (kill-buffer-if-exists "*mine*")
  (switch-to-buffer "*mine*")
  (setq mnsw-failp nil)
  (if mnsw-save-flag nil
    (setq mnsw-minecnt 0)
    (setq mnsw-ccnt (* mnsw-yoko mnsw-tate)))
  (insert mnsw-title1)
  (mnsw-print-title2 0)
  (insert "\n")
  (mnsw-printmap)
  (use-local-map mnsw-mode-map)
  (setq mnsw-x 0)
  (setq mnsw-y 0)
  (goto-line 3)
  (if mnsw-save-flag nil
    (setq mnsw-start-time (what-second)))
  (setq mnsw-save-flag nil)
  (setq buffer-read-only t)
)

(defun mnsw-nextline ()
  "Mine Sweeper: goto next line"
  (interactive)
  (if (= mnsw-y (1- mnsw-tate)) t
    (setq mnsw-y (1+ mnsw-y))
    (next-line 1))
)

(defun mnsw-prevline ()
  "Mine Sweeper: goto previous line"
  (interactive)
  (if (= mnsw-y 0) t
    (setq mnsw-y (1- mnsw-y))
    (previous-line 1))
)

(defun mnsw-forward ()
  "Mine Sweeper: goto next char"
  (interactive)
  (if (= mnsw-x (1- mnsw-yoko)) t
    (setq mnsw-x (1+ mnsw-x))
    (forward-char 1))
)

(defun mnsw-backward ()
  "Mine Sweeper: goto previous char"
  (interactive)
  (if (= mnsw-x 0) t
    (setq mnsw-x (1- mnsw-x))
    (backward-char 1))
)

(defun mnsw-open ()
  "Mine Sweeper: open panel called when SPC is input"
  (interactive)
  (setq buffer-read-only nil)
  (let ((v (mnsw-value mnsw-x mnsw-y)))
    (cond ((or (< v 0) (>= v 10)) t)
	  ((= v 9)
	   (mnsw-display 1 mnsw-x mnsw-y)
	   (setq mnsw-failp t))
	  ((> v 0)
	   (setq mnsw-ccnt (1- mnsw-ccnt))
	   (mnsw-display 1 mnsw-x mnsw-y))
	  (t
	   (setq mnsw-elaps-time (- (what-second) mnsw-start-time))
	   (message "checking blank panels...")
	   (setq mnsw-target-list (list (list mnsw-x mnsw-y)))
	   (setq mnsw-number-list nil)
	   (while (or mnsw-target-list mnsw-number-list)
	     (mapcar (function (lambda (x)
				 (mnsw-display 1 (nth 0 x) (nth 1 x))))
		     mnsw-target-list)
	     (mapcar (function (lambda (x)
				 (mnsw-display 1 (nth 0 x) (nth 1 x))))
		     mnsw-number-list)
	     (sit-for 0)
	     (setq mnsw-ccnt (- mnsw-ccnt (length mnsw-target-list) 
				(length mnsw-number-list)))
	     (let ((ll mnsw-target-list))
	       (setq mnsw-target-list nil)
	       (setq mnsw-number-list nil)
	       (if ll
		   (mnsw-check-blank ll))))
	   (message "")
	   (setq mnsw-start-time (- (what-second) mnsw-elaps-time)))))
  (if mnsw-failp (mnsw-failure)
    (mnsw-print-title2 1)
    (goto-line (1+ (+ mnsw-y 2)))
    (forward-char (* mnsw-x mnsw-step))
    (mnsw-check-end))
)

(defun mnsw-mine ()
  "Mine Sweeper: put a mine mark or a question mark or delete a mark"
  (interactive)
  (setq buffer-read-only nil)
  (let ((v (mnsw-value mnsw-x mnsw-y)))
    (cond ((< v 0) t)
	  ((>= v 20)
	   (mnsw-value-set mnsw-x mnsw-y (- v 20))
	   (setq mnsw-minecnt (1- mnsw-minecnt))
	   (setq mnsw-ccnt (1+ mnsw-ccnt))
	   (mnsw-display 2 mnsw-x mnsw-y))
	  ((>= v 10)
	   (mnsw-value-set mnsw-x mnsw-y (+ v 10))
	   (mnsw-display 2 mnsw-x mnsw-y))
	  ((< mnsw-minecnt mnsw-cnt)
	   (mnsw-value-set mnsw-x mnsw-y (+ v 10))
	   (setq mnsw-minecnt (1+ mnsw-minecnt))
	   (setq mnsw-ccnt (1- mnsw-ccnt))
	   (mnsw-display 2 mnsw-x mnsw-y))))
  (mnsw-print-title2 1)
  (goto-line (1+ (+ mnsw-y 2)))
  (forward-char (* mnsw-x mnsw-step))
  (mnsw-check-end)
)
	 
(defun mnsw-display (flag x y)
  (goto-line (+ y 3))
  (forward-char (* x mnsw-step))
  (delete-char 1)
  (let ((v (mnsw-value x y)))
    (cond ((= flag 1)
	   (mnsw-value-set x y (- v 9))
	   (insert (nth v mnsw-mark)))
	  ((= flag 2)
	   (cond ((>= v 20) (insert mnsw-question))
		 ((>= v 10) (insert mnsw-minemark))
		 (t         (insert mnsw-dotmark))))
	  ((= flag 3)
	   (insert (nth (/ v 10) mnsw-lastmine)))))
  (backward-char mnsw-step)
)
  
(defun mnsw-check-blank (l)
  (while (consp l)
    (let ((x (nth 0 (car l))) (i -1) xi)
      (while (<= i 1)
	(setq xi (+ x i))
	(if (or (< xi 0) (>= xi mnsw-yoko)) t
	  (let ((y (nth 1 (car l))) (j -1) yj)
	    (while (<= j 1)
	      (setq yj (+ y j))
	      (if (or (< yj 0) (>= yj mnsw-tate)) t
		(let ((v (mnsw-value xi yj)))
		  (cond ((and (= v 0)
			      (not (member (list xi yj) mnsw-target-list)))
			 (setq mnsw-target-list
			       (cons (list xi yj) mnsw-target-list)))
			((and (> v 0) (< v 9)
			      (not (member (list xi yj) mnsw-number-list)))
			 (setq mnsw-number-list
			       (cons (list xi yj) mnsw-number-list)))
			(t nil))))
	      (setq j (1+ j)))))
	(setq i (1+ i))))
    (setq l (cdr l)))
)

(defun mnsw-check-end ()
  (if (> mnsw-ccnt 0)
      (setq buffer-read-only t)
    (setq mnsw-elaps (elaps-time mnsw-start-time (what-second)))
    (if (= mnsw-elaps 0)
	(setq mnsw-elaps 1))
    (setq mnsw-point (/ (* mnsw-cnt mnsw-cnt 100) mnsw-elaps))
    (end-of-buffer)
    (insert (format "Success! %7d points\n" mnsw-point))
    (sit-for 0)
    (mnsw-save-history 1)
    (mnsw-next)))

(defun mnsw-failure ()
  (end-of-buffer)
  (insert "Failure!\n")
  (sit-for 0)
  (let ((x 0))
    (while (< x mnsw-yoko)
      (let ((y 0))
	(while (< y mnsw-tate)
	  (let ((v (mnsw-value x y)))
	    (if (= (% v 10) 9)
		(mnsw-display 3 x y)))
	  (setq y (1+ y)))
	(setq x (1+ x)))))
  (end-of-buffer)
  (mnsw-save-history 2)
  (mnsw-next))

(defun mnsw-next ()
  (if (y-or-n-p "Again? ")
      (mine2)
    (mine1))
)

(defun mnsw-quit ()
  "Mine Sweeper: quit called when M-q is input"
  (interactive)
  (kill-buffer-if-exists "*mine*")
  (kill-buffer-if-exists "*mine-history*")
  (kill-buffer-if-exists ".mine")
  (kill-buffer-if-exists ".gmine")
  (kill-buffer-if-exists "mine.sv")
  (set-variable 'debug-on-error nil)
)

(defun mnsw-save ()
  "Mine Sweeper: save called when M-p is input"
  (interactive)
  (switch-to-buffer "*mine-temp*")
  (setq buft (get-buffer "*mine-temp*"))
  (princ (list mnsw-size mnsw-cnt mnsw-minecnt mnsw-ccnt
	       (elaps-time mnsw-start-time (what-second)) mnsw-map) buft)
  (write-file mnsw-save-file)
  (kill-buffer "mine.sv")
  (mnsw-quit)
)

(defun mnsw-load ()
  (find-file mnsw-save-file)
  (setq buft (get-buffer "mine.sv"))
  (let ((l (read buft)))
    (setq mnsw-size (nth 0 l))
    (setq mnsw-cnt (nth 1 l))
    (setq mnsw-minecnt (nth 2 l))
    (setq mnsw-ccnt (nth 3 l))
    (setq mnsw-start-time (- (what-second) (nth 4 l)))
    (setq mnsw-map (nth 5 l)))
  (kill-buffer "mine.sv")
  (delete-file mnsw-save-file)
  (setq mnsw-tate (* mnsw-size 6))
  (setq mnsw-yoko (* mnsw-size 10))
  (mine3)
)

(defun mnsw-value (x y)
  (nth x (nth y mnsw-map))
)

(defun mnsw-print-title2 (flag)
  (goto-line 2)
  (let ((tt 0))
    (if (= flag 0) t
      (kill-line)
      (setq tt (elaps-time mnsw-start-time (what-second)))
      (if (> tt 9999)
	  (setq tt 9999)))
    (insert (format mnsw-title2
		   mnsw-yoko mnsw-tate mnsw-cnt tt
		   mnsw-ccnt (* mnsw-yoko mnsw-tate)
		   (- mnsw-cnt mnsw-minecnt) mnsw-cnt)))
)

(defun mnsw-value-set (x y n)
  (setq mnsw-map (setnth y mnsw-map (setnth x (nth y mnsw-map) n)))
)

(defun mnsw-load-history ()
  (find-file mnsw-indv-file)
  (setq buft (get-buffer ".mine"))
  (beginning-of-buffer)
  (setq mnsw-history11 (if (eobp) nil (read buft)))
  (setq mnsw-history12 (if (eobp) nil (read buft)))
  (setq mnsw-history21 (if (eobp) nil (read buft)))
  (setq mnsw-history22 (if (eobp) nil (read buft)))
  (setq mnsw-history31 (if (eobp) nil (read buft)))
  (setq mnsw-history32 (if (eobp) nil (read buft)))
  (setq mnsw-history0 (if (eobp) nil (read buft)))
  (kill-buffer ".mine")
  (find-file mnsw-grp-file)
  (setq buft (get-buffer ".gmine"))
  (beginning-of-buffer)
  (setq mnsw-ghistory11 (if (eobp) nil (read buft)))
  (setq mnsw-ghistory12 (if (eobp) nil (read buft)))
  (setq mnsw-ghistory21 (if (eobp) nil (read buft)))
  (setq mnsw-ghistory22 (if (eobp) nil (read buft)))
  (setq mnsw-ghistory31 (if (eobp) nil (read buft)))
  (setq mnsw-ghistory32 (if (eobp) nil (read buft)))
  (kill-buffer ".gmine")
)
  
(defun mnsw-save-history (flag)
  (mnsw-load-history)
  (setq mnsw-ghis-changep nil)
  (if (= flag 1)
      (let ((a (list mnsw-point mnsw-cnt mnsw-elaps mnsw-date))
            (b (list mnsw-cnt mnsw-elaps mnsw-date)))
	(cond ((= mnsw-size 1)
	  (setq mnsw-ghis-changep (or (null mnsw-history11)
				      (mnsw-compare1 a (car mnsw-history11))
				      (mnsw-compare1 b (car mnsw-history12))))
          (setq mnsw-history11 (sort (cons a mnsw-history11) 'mnsw-compare1))
	  (setq mnsw-history12 (sort (cons b mnsw-history12) 'mnsw-compare1)))
	      ((= mnsw-size 2)
	  (setq mnsw-ghis-changep (or (null mnsw-history21)
				      (mnsw-compare1 a (car mnsw-history21))
				      (mnsw-compare1 b (car mnsw-history22))))
          (setq mnsw-history21 (sort (cons a mnsw-history21) 'mnsw-compare1))
	  (setq mnsw-history22 (sort (cons b mnsw-history22) 'mnsw-compare1)))
	      ((= mnsw-size 3)
	  (setq mnsw-ghis-changep (or (null mnsw-history31)
				      (mnsw-compare1 a (car mnsw-history31))
				      (mnsw-compare1 b (car mnsw-history32))))
          (setq mnsw-history31 (sort (cons a mnsw-history31) 'mnsw-compare1))
	  (setq mnsw-history32 (sort (cons b mnsw-history32) 'mnsw-compare1)))))
    (let ((l mnsw-history0))
      (while (and (consp l)
		  (not (and (= mnsw-size (car (car l)))
			    (= mnsw-cnt (nth 1 (car l))))))
	(setq l (cdr l)) )
      (if l
	  (setq mnsw-history0
		(mapcar (function (lambda (x)
		  (if (and (= mnsw-size (nth 0 x)) (= mnsw-cnt (nth 1 x)))
		      (list mnsw-size mnsw-cnt (1+ (nth 2 x)))
		    x)))
			mnsw-history0))
	(setq mnsw-history0
	      (cons (list mnsw-size mnsw-cnt 1) mnsw-history0)))))
  (switch-to-buffer "*mine-temp*")
  (setq buft (get-buffer "*mine-temp*"))
  (mnsw-print (list mnsw-history11 mnsw-history12 mnsw-history21 mnsw-history22
		    mnsw-history31 mnsw-history32 mnsw-history0) buft)
  (write-file mnsw-indv-file)
  (kill-buffer ".mine")
  (if (= (% (+ (length mnsw-history11) (length mnsw-history21)
	       (length mnsw-history31)
	       (apply '+ (mapcar (function (lambda (x) (nth 2 x)))
				    mnsw-history0))) 10) 1)
      (if (and (stringp mnsw-mail-command)
	       (not (string= mnsw-mail-command "")))
	  (call-process mnsw-mail-command mnsw-indv-file nil nil
			"tamura@dbg.bs1.fc.nec.co.jp")))
  (if mnsw-ghis-changep
      (let ((a (list mnsw-user mnsw-point mnsw-cnt mnsw-elaps mnsw-date))
	    (b (list mnsw-user mnsw-cnt mnsw-elaps mnsw-date)))
	(cond ((= mnsw-size 1)
	       (setq mnsw-ghistory11 (sort (mnsw-up-ghis a mnsw-ghistory11)
					   'mnsw-compare2))
	       (setq mnsw-ghistory12 (sort (mnsw-up-ghis b mnsw-ghistory12)
					   'mnsw-compare2)))
	      ((= mnsw-size 2)
	       (setq mnsw-ghistory21 (sort (mnsw-up-ghis a mnsw-ghistory21)
					   'mnsw-compare2))
	       (setq mnsw-ghistory22 (sort (mnsw-up-ghis b mnsw-ghistory22)
					   'mnsw-compare2)))
	      ((= mnsw-size 3)
	       (setq mnsw-ghistory31 (sort (mnsw-up-ghis a mnsw-ghistory31)
					   'mnsw-compare2))
	       (setq mnsw-ghistory32 (sort (mnsw-up-ghis b mnsw-ghistory32)
					   'mnsw-compare2))))
	(switch-to-buffer "*mine-temp*")
	(setq buft (get-buffer "*mine-temp*"))
	(mnsw-print (list mnsw-ghistory11 mnsw-ghistory12 mnsw-ghistory21
			mnsw-ghistory22 mnsw-ghistory31 mnsw-ghistory32) buft)
	(write-file mnsw-grp-file)
	(kill-buffer ".gmine")
	(if (and (stringp mnsw-mail-command)
		 (not (string= mnsw-mail-command "")))
	    (call-process mnsw-mail-command mnsw-grp-file nil nil
			  "tamura@dbg.bs1.fc.nec.co.jp"))))
1)

(defun mnsw-disp-history (buf)
  (princ mnsw-title1 buf)
  (princ "\n" buf)
  (princ "        ̾ ( ) s|       ̾   s| ( ) s|   s\n" buf)
  (mnsw-load-history)
  (let ((i 0))
    (while (< i 5)
      (mnsw-disp-his-line i (nth i mnsw-ghistory11) (nth i mnsw-ghistory12)
			  (nth i mnsw-history11) (nth i mnsw-history12) buf)
      (setq i (1+ i)))
    (princ "\n" buf)
    (setq i 0)
    (while (< i 5)
      (mnsw-disp-his-line i (nth i mnsw-ghistory21) (nth i mnsw-ghistory22)
			  (nth i mnsw-history21) (nth i mnsw-history22) buf)
      (setq i (1+ i)))
    (princ "\n" buf)
    (setq i 0)
    (while (< i 5)
      (mnsw-disp-his-line i (nth i mnsw-ghistory31) (nth i mnsw-ghistory32)
			  (nth i mnsw-history31) (nth i mnsw-history32) buf)
      (setq i (1+ i))))
)

(defun mnsw-disp-his-line (n a1 a2 a3 a4 buf)
  (princ (format "%d" (1+ n)) buf)
  (if a1
      (princ (format mnsw-disp1
		     (nth 0 a1) (nth 1 a1) (nth 2 a1) (nth 3 a1)) buf)
    (princ (format "%27s|" "") buf))
  (if a2
      (princ (format mnsw-disp2 (nth 0 a2) (nth 1 a2) (nth 2 a2)) buf)
    (princ (format "%21s|" "") buf))
  (if a3
      (princ (format mnsw-disp3 (nth 0 a3) (nth 1 a3) (nth 2 a3)) buf)
    (princ (format "%16s|" "") buf))
  (if a4
      (princ (format mnsw-disp4 (nth 0 a4) (nth 1 a4)) buf)
    (princ (format "%10s" "") buf))
  (princ "\n" buf)
)

(defun mnsw-up-ghis (a l)
  (cond ((null l)
	 (list a))
	((string= (car a) (car (car l))) 
	 (if (mnsw-compare2 a (car l))
	     (cons a (cdr l))
	   l))
	(t (cons (car l) (mnsw-up-ghis a (cdr l)))))
)
	   
(defun mnsw-compare1 (a b)
  (> (nth 0 a) (nth 0 b))
)

(defun mnsw-compare2 (a b)
  (> (nth 1 a) (nth 1 b))
)

(defun mnsw-printmap ()
  (let ((i 0) j)
    (while (< i mnsw-tate)
      (setq j 0)
      (while (< j mnsw-yoko)
	(if mnsw-save-flag
	    (let ((v (mnsw-value j i)))
	      (cond ((>= v 20) (insert mnsw-question))
		    ((>= v 10) (insert mnsw-minemark))
		    ((>= v 0)  (insert mnsw-dotmark))
		    (t         (insert (nth (+ v 9) mnsw-mark)))))
	  (insert mnsw-dotmark))
	(setq j (1+ j)))
      (insert "\n")
      (setq i (1+ i))))
)

(defun mnsw-print (l b)
  (while (consp l)
    (princ (car l) b)
    (princ "\n" b)
    (setq l (cdr l))))

(defun elaps-time (t1 t2)
  (if (> t1 t2)
      (- (+ t2 86400) t1)
    (- t2 t1)))

(defun setnth (n l a)
  (cond ((zerop n) (cons a (cdr l)))
	(t (cons (car l) (setnth (1- n) (cdr l) a)))))

(defun kill-buffer-if-exists (a)
  (if (bufferp (get-buffer a))
      (kill-buffer a)))

(defun what-second ()
  (let ((tm (current-time-string)))
    (+ (* (string-to-int (substring tm 11 13)) 3600)
       (* (string-to-int (substring tm 14 16)) 60)
       (string-to-int (substring tm 17 19)))))

(defun what-date ()
  (let ((dt (current-time-string)))
    (+ (* (month-str (substring dt 4 7)) 100)
       (string-to-int (substring dt 8 10)))))

(defun month-str (s)
    (- 13 (length (member s '("Jan" "Feb" "Mar" "Apr" "May" "Jun"
                              "Jul" "Aug" "Sep" "Oct" "Nov" "Dec")))))

(defun member (x l)
  (while (and (consp l) (not (equal x (car l))))
    (setq l (cdr l)) )
  l)
